﻿Public Class Game

    Private chip As chipPictureBox
    Private btnSpin As Button
    Private btnNew As Button
    Private display As PictureBox
    Private squares As New List(Of Square)
    Private currentPosition As Integer = 0
    Private r As Random
    Private indicesUsed As New List(Of Integer)
    Private sounds1 As New Media.SoundPlayer(My.Resources.floop)
    Private sounds2 As New Media.SoundPlayer(My.Resources.back)

    Private _questionType As Integer
    Public Property questionType() As Integer
        Get
            Return _questionType
        End Get
        Set(ByVal value As Integer)
            _questionType = value
        End Set
    End Property

    ''' <summary>
    ''' Constructor
    ''' </summary>
    ''' <param name="r"></param>
    ''' <param name="chip"></param>
    ''' <param name="spinButton"></param>
    ''' <param name="newButton"></param>
    ''' <param name="spinnerPicturebox"></param>
    ''' <remarks>Sets up squares List</remarks>
    Public Sub New(ByVal r As Random, ByVal chip As chipPictureBox, ByVal spinButton As Button, ByVal newButton As Button, ByVal spinnerPicturebox As PictureBox)
        Me.chip = chip
        Me.btnSpin = spinButton
        Me.btnNew = newButton
        Me.display = spinnerPicturebox
        Me.r = r
        For x As Integer = 0 To 80
            Dim ir As Integer = 8 - CInt(Math.Floor(x / 9))
            Dim ic As Integer = If(ir Mod 2 = 1, 8 - (x Mod 9), x Mod 9)
            squares.Add(New Square With {.centrePoint = New Point(12 + 1 + 32 + (ic * 64), 12 + 1 + 32 + (ir * 64))})
        Next
        Me.chip.Location = New Point(squares(0).centrePoint.X - 25, squares(0).centrePoint.Y - 25)
    End Sub

    ''' <summary>
    ''' newGame
    ''' </summary>
    ''' <param name="gr"></param>
    ''' <param name="f"></param>
    ''' <remarks>Resets the game.</remarks>
    Public Sub newGame(ByVal gr As Graphics, ByVal f As Font)
        Dim qSquares As New List(Of Integer)
        qSquares.Add(81)

        For x As Integer = 1 To 17
            Dim s As Integer = r.Next(2, 81)
            If Not Enumerable.Range(s, 3).Any(Function(i) qSquares.Contains(i)) AndAlso Not Enumerable.Range(s - 3, 3).Any(Function(i) qSquares.Contains(i)) Then
                qSquares.Add(s)
            Else
                x -= 1
            End If
        Next

        For x As Integer = 0 To 80
            squares(x).isQSquare = qSquares.Contains(x + 1)
            If squares(x).isQSquare Then
                Dim ir As Integer = 8 - CInt(Math.Floor(x / 9))
                Dim ic As Integer = If(ir Mod 2 = 1, 8 - (x Mod 9), x Mod 9)
                Dim sr As New Rectangle(1 + (ic * 64), 1 + (ir * 64), 64, 64)
                Dim sf As New StringFormat
                sf.Alignment = StringAlignment.Center
                sf.LineAlignment = StringAlignment.Center
                Dim cs As Color = Color.Silver
                gr.DrawString("?", New Font(f.FontFamily, 50, FontStyle.Bold), New SolidBrush(Color.FromArgb(150, cs.R, cs.G, cs.B)), sr, sf)
            End If
        Next

        Me.chip.Location = New Point(squares(0).centrePoint.X - 25, squares(0).centrePoint.Y - 25)
        currentPosition = 0
    End Sub

    ''' <summary>
    ''' moveChip
    ''' </summary>
    ''' <param name="moves"></param>
    ''' <param name="p"></param>
    ''' <remarks>Moves the ChipPictureBox the specified number of moves.
    ''' Moves can be positive or negative.</remarks>
    Public Sub moveChip(ByVal moves As Integer, ByVal p As frmGame)
        Dim movesleft As Integer = moves
        If moves > 0 Then
            display.Image = movesIndicator(moves.ToString)
            display.Refresh()
            For x As Integer = currentPosition + 1 To currentPosition + moves
                Dim ir As Integer = 8 - CInt(Math.Floor(x / 9))
                Dim offset As Integer = If(ir Mod 2 = 1, -50, 0)
                If squares(x - 1).centrePoint.Y = squares(x).centrePoint.Y Then
                    Me.chip.Location = New Point(squares(x - 1).centrePoint.X + offset, squares(x).centrePoint.Y - 25)
                    p.Refresh()
                    Threading.Thread.Sleep(250)
                    Me.chip.Location = New Point(squares(x).centrePoint.X - 25, squares(x).centrePoint.Y - 25)
                    p.Refresh()
                    sounds1.Play()
                    Threading.Thread.Sleep(250)
                    currentPosition += 1
                    movesleft -= 1
                    display.Image = movesIndicator(movesleft.ToString)
                    display.Refresh()
                    If squares(currentPosition).isQSquare Then
                        sounds1.Play()
                        showQuestion(movesleft, p)
                        Return
                    End If
                Else
                    Me.chip.Location = New Point(squares(x - 1).centrePoint.X - 25, squares(x).centrePoint.Y)
                    p.Refresh()
                    Threading.Thread.Sleep(250)
                    Me.chip.Location = New Point(squares(x - 1).centrePoint.X - 25, squares(x).centrePoint.Y - 25)
                    p.Refresh()
                    sounds1.Play()
                    Threading.Thread.Sleep(250)
                    currentPosition += 1
                    movesleft -= 1
                    display.Image = movesIndicator(movesleft.ToString)
                    display.Refresh()
                    If squares(currentPosition).isQSquare Then
                        sounds1.Play()
                        showQuestion(movesleft, p)
                        Return
                    End If
                End If
            Next
        Else
            display.Image = movesIndicator("0")
            display.Refresh()
            For x As Integer = currentPosition - 1 To currentPosition + moves Step -1
                Dim ir As Integer = 8 - CInt(Math.Floor(x / 9))
                Dim offset As Integer = If(ir Mod 2 = 1, -50, 0)
                If squares(x + 1).centrePoint.Y = squares(x).centrePoint.Y Then
                    Me.chip.Location = New Point(squares(x).centrePoint.X + offset, squares(x).centrePoint.Y - 25)
                    p.Refresh()
                    Threading.Thread.Sleep(250)
                    Me.chip.Location = New Point(squares(x).centrePoint.X - 25, squares(x).centrePoint.Y - 25)
                    p.Refresh()
                    sounds2.Play()
                    Threading.Thread.Sleep(250)
                    currentPosition -= 1
                    If squares(currentPosition).isQSquare Then
                        sounds1.Play()
                        showQuestion(0, p)
                        Return
                    End If
                Else
                    Me.chip.Location = New Point(squares(x).centrePoint.X - 25, squares(x + 1).centrePoint.Y)
                    p.Refresh()
                    Threading.Thread.Sleep(250)
                    Me.chip.Location = New Point(squares(x).centrePoint.X - 25, squares(x).centrePoint.Y - 25)
                    p.Refresh()
                    sounds2.Play()
                    Threading.Thread.Sleep(250)
                    currentPosition -= 1
                    If squares(currentPosition).isQSquare Then
                        sounds1.Play()
                        showQuestion(0, p)
                        Return
                    End If
                End If
            Next
        End If
        Me.btnSpin.Enabled = True
    End Sub

    ''' <summary>
    ''' showQuestion
    ''' </summary>
    ''' <param name="moves"></param>
    ''' <param name="p"></param>
    ''' <remarks>Determines whether a Grammar or a Maths question should be asked.
    ''' Responds to answers (in the form of dialogresults).</remarks>
    Private Sub showQuestion(ByVal moves As Integer, ByVal p As frmGame)
        Dim yesScore As Integer = moves * 2 + 1
        Dim noScore As Integer = -getMoves()
        Dim passScore As Integer = moves
        Dim t As Integer = r.Next(0, 2)

        Dim response As DialogResult

        Dim frm As Form = Nothing
        If questionType = 0 OrElse (questionType = 2 AndAlso t = 0) Then
            frm = New frmGrammar(r, indicesUsed)
            response = frm.ShowDialog
        ElseIf questionType = 1 OrElse (questionType = 2 AndAlso t = 1) Then
            frm = New frmMaths(r)
            response = frm.ShowDialog
        End If
        Dim f As frmGrammar = TryCast(frm, frmGrammar)
        If Not f Is Nothing Then
            Me.indicesUsed = f.indicesUsed
        End If
        If response = Windows.Forms.DialogResult.Yes Then
            If currentPosition = 80 Then
                sounds1.PlayLooping()
                Threading.Thread.Sleep(600)
                sounds1.Stop()
                MsgBox("You've done it. You won!", MsgBoxStyle.Information, "Game over")
                Me.btnSpin.Enabled = False
                Me.btnNew.Enabled = True
                Me.display.Image = Nothing
                Return
            End If
            moveChip(yesScore, p)
        ElseIf response = Windows.Forms.DialogResult.No Then
            moveChip(noScore, p)
        ElseIf response = Windows.Forms.DialogResult.Cancel Then
            If passScore > 0 Then
                If currentPosition < 80 Then
                    moveChip(passScore, p)
                Else
                    showQuestion(0, p)
                End If
            Else
                showQuestion(0, p)
            End If
        End If
    End Sub

    ''' <summary>
    ''' Finds last question square. If none, finds moves to start square.
    ''' </summary>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Private Function getMoves() As Integer
        If currentPosition = 0 Then Return 0
        Dim counter As Integer = 0
        For x As Integer = currentPosition - 1 To 0 Step -1
            counter += 1
            If squares(x).isQSquare Then
                Exit For
            End If
        Next
        Return counter
    End Function

    ''' <summary>
    ''' Displays remaining moves
    ''' </summary>
    ''' <param name="moves"></param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Private Function movesIndicator(ByVal moves As String) As Bitmap
        Dim ts As SizeF = TextRenderer.MeasureText(moves, New Font("Arial", 72))
        Dim img As New Bitmap(CInt(ts.Width), 100)
        Dim gr As Graphics = Graphics.FromImage(img)
        gr.DrawString(moves, New Font("Arial", 72), New SolidBrush(Color.FromArgb(65, 163, 219)), 0, 0)
        Return img
    End Function

End Class
